/*
 * Java
 *
 * Copyright 2015-2021 MicroEJ Corp. All rights reserved.
 * This library is provided in source code for use, modification and test, subject to license terms.
 * Any modification of the source code will break MicroEJ Corp. warranties on the whole library.
 */
package ej.net;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import android.net.ConnectivityManager;
import android.net.NetworkCapabilities;
import android.net.NetworkInfo;
import ej.annotation.Nullable;
import ej.bon.Timer;
import ej.bon.TimerTask;
import ej.net.util.NetUtil;
import ej.util.message.Level;

/**
 * Basic full java implementation. Consider the network is connected when there is at least one {@link NetworkInterface}
 * which address is not <code>0.0.0.0</code>.
 */
public class PollerConnectivityManager extends ConnectivityManager {

	/**
	 * Capability with internet access.
	 */
	protected static final NetworkCapabilities INTERNET_CONNECTED = new NetworkCapabilities(
			new int[] { NetworkCapabilities.NET_CAPABILITY_INTERNET });

	/**
	 * Capability with internet access.
	 */
	protected static final NetworkCapabilities INTERNET_UNCONNECTED = new NetworkCapabilities();

	private static final boolean DEBUG = false;
	private final Timer timer;
	private final Map<String, Network> networks;

	/**
	 * Base property name.
	 */
	protected static final String BASE_PROPERTY = "connectivity."; //$NON-NLS-1$
	private static final String TASK = "task."; //$NON-NLS-1$
	/**
	 * Default delay is 0 : means start polling ASAP.
	 */
	private static final long DEFAULT_DELAY = Long.getLong(BASE_PROPERTY + TASK + "delay", 0).longValue(); //$NON-NLS-1$

	/**
	 * Default period when there's no connectivity.
	 * <p>
	 * Likely to be shorter than the {@link DEFAULT_PERIOD_WHEN_AVAILABLE} as we want to detect the network availability
	 * as soon as possible.
	 */
	private static final long DEFAULT_PERIOD_WHEN_LOST = Long.getLong(BASE_PROPERTY + TASK + "period", 1000) //$NON-NLS-1$
			.longValue();

	/**
	 * Default period when there's connectivity.
	 * <p>
	 * Likely to be longer than the {@link DEFAULT_PERIOD_WHEN_LOST}, as it's likely that the network will not go down,
	 * and it's acceptable not to detect it immediately.
	 */
	private static final long DEFAULT_PERIOD_WHEN_AVAILABLE = Long
			.getLong(BASE_PROPERTY + TASK + "period_when_up", 5 * DEFAULT_PERIOD_WHEN_LOST).longValue(); //$NON-NLS-1$

	/**
	 * Default period when there is internet connectivity.
	 * <p>
	 * Likely to be longer than the {@link DEFAULT_PERIOD_WHEN_AVAILABLE}, as it's likely that the internet will not go
	 * down, and it's acceptable not to detect it immediately.
	 */
	private static final long DEFAULT_PERIOD_WHEN_INTERNET = Long
			.getLong(BASE_PROPERTY + TASK + "period_when_internet", 4 * DEFAULT_PERIOD_WHEN_AVAILABLE).longValue(); //$NON-NLS-1$
	/* package */ static final String EMPTY_NETWORK = ""; //$NON-NLS-1$

	private final long periodWhenLost;
	private final long periodWhenAvailable;
	private final long periodWhenInternet;
	private @Nullable ConnectivityManagerTimerTask pollerTask;
	private Network activeNetwork;
	private long currentPeriod;

	/**
	 * Instantiates a {@link PollerConnectivityManager} with default periods and delay and a new timer.
	 */
	public PollerConnectivityManager() {
		this(new Timer());
		this.timer.schedule(new TimerTask() {

			@Override
			public void run() {
				Thread.currentThread().setName(PollerConnectivityManager.class.getSimpleName());
			}
		}, 0);
	}

	/**
	 * Instantiates a {@link PollerConnectivityManager} with default value.
	 *
	 * @param timer
	 *            the timer to use, cannot be <code>null</code>.
	 */
	public PollerConnectivityManager(Timer timer) {
		this(timer, DEFAULT_DELAY, DEFAULT_PERIOD_WHEN_AVAILABLE, DEFAULT_PERIOD_WHEN_LOST,
				DEFAULT_PERIOD_WHEN_INTERNET);
	}

	/**
	 * Instantiates a {@link PollerConnectivityManager} with the same period when up or not.
	 *
	 * @param timer
	 *            the timer to use, cannot be <code>null</code>.
	 * @param delay
	 *            the delay before the start.
	 * @param period
	 *            the period for the polling.
	 */
	public PollerConnectivityManager(Timer timer, long delay, long period) {
		this(timer, delay, period, period);
	}

	/**
	 * Instantiates a {@link PollerConnectivityManager} with the different periods when available or not. The same
	 * period is used wth or without internet access when availble.
	 *
	 * @param timer
	 *            the timer to use, cannot be <code>null</code>.
	 * @param delay
	 *            the delay before the start.
	 * @param periodWhenAvailable
	 *            the period for the polling when at least one interface is available.
	 * @param periodWhenLost
	 *            the period for the polling when all the interface are down.
	 */
	public PollerConnectivityManager(Timer timer, long delay, long periodWhenAvailable, long periodWhenLost) {
		this(timer, delay, periodWhenAvailable, periodWhenLost, periodWhenAvailable);
	}

	/**
	 * Instantiates a {@link PollerConnectivityManager} with the same period when connected or not.
	 *
	 * @param timer
	 *            the timer to use, cannot be <code>null</code>.
	 * @param delay
	 *            the delay before the start.
	 * @param periodWhenAvaialble
	 *            the period for the polling when at least one interface is available.
	 * @param periodWhenLost
	 *            the period for the polling when all the interface are down.
	 * @param periodWhenInternet
	 *            the period for the polling when there is internet access.
	 */
	public PollerConnectivityManager(Timer timer, long delay, long periodWhenAvaialble, long periodWhenLost,
			long periodWhenInternet) {
		this.networks = new HashMap<>();
		Network network = new Network(EMPTY_NETWORK);
		this.networks.put(EMPTY_NETWORK, network);
		this.timer = timer;
		this.periodWhenLost = periodWhenLost;
		this.periodWhenAvailable = periodWhenAvaialble;
		this.periodWhenInternet = periodWhenInternet;
		this.activeNetwork = network;
		setActiveNetwork(network);
		NetworkInfo activeNetworkInfo = getActiveNetworkInfo();
		if (activeNetworkInfo != null) {
			setAvailable(false, network, activeNetworkInfo);
		}
		startPolling(delay, periodWhenLost);
	}

	/**
	 * Starts the polling function.
	 *
	 * @param delay
	 *            the delay before starting.
	 */
	public void startPolling(long delay) {
		startPolling(delay, this.periodWhenLost);
	}

	/**
	 * Stops the polling function.
	 */
	public synchronized void stopPolling() {
		if (this.pollerTask != null) {
			this.pollerTask.cancel();
			this.pollerTask = null;
		}
	}

	/**
	 * Cancels the associated timer.
	 */
	public void cancel() {
		stopPolling();
		this.timer.cancel();
	}

	/**
	 * Logs a message.
	 *
	 * @param msg
	 *            the message to log.
	 */
	protected void log(int msg) {
		if (DEBUG) {
			ConnectivityMessages.LOGGER.log(Level.INFO, ConnectivityMessages.CATEGORY, msg);
		}
	}

	/**
	 * Logs a message and a throwable.
	 *
	 * @param msg
	 *            the message to log.
	 * @param t
	 *            a throwable to print, cannot be <code>null</code>.
	 */
	protected void log(int msg, Throwable t) {
		if (DEBUG) {
			ConnectivityMessages.LOGGER.log(Level.INFO, ConnectivityMessages.CATEGORY, msg, t);
		}
	}

	private synchronized void startPolling(long delay, long period) {
		stopPolling();
		this.currentPeriod = period;
		this.pollerTask = new ConnectivityManagerTimerTask();
		this.timer.schedule(this.pollerTask, delay, this.currentPeriod);
	}

	private class ConnectivityManagerTimerTask extends TimerTask {

		@Override
		public void run() {
			log(ConnectivityMessages.TRY);

			String activenetworkInterface = PollerConnectivityManager.this.activeNetwork.getNetworkInterface();
			try {
				// If still connected to internet.
				if (PollerConnectivityManager.this.activeNetwork.getCapabilities() == INTERNET_CONNECTED) {
					InetAddress checkConnected = checkConnected(getNetworkInterface(activenetworkInterface));
					if (checkConnected != null && getCapabilities(checkConnected) == INTERNET_CONNECTED) {
						log(ConnectivityMessages.SKIP);
						return;
					}
				}
			} catch (SocketException e2) {
				log(ConnectivityMessages.ERROR_UNKNOWN, e2);
			}

			Enumeration<NetworkInterface> e;
			try {
				e = getNetworkInterfaces();
				checkNetworkInterfacesState(e);
			} catch (Throwable e1) {
				log(ConnectivityMessages.ERROR_UNKNOWN, e1);
			}
		}

		private void checkNetworkInterfacesState(@Nullable Enumeration<NetworkInterface> interfaces)
				throws SocketException {
			Network activeNetwork = PollerConnectivityManager.this.activeNetwork;
			Network internetNetwork = null;
			Network availableNetwork = null;
			Network notAvailableNetwork = null;
			Network downNetwork = null;

			for (Entry<String, Network> network : PollerConnectivityManager.this.networks.entrySet()) {
				network.getValue().setPresent(false);
			}

			if (interfaces != null) {
				while (interfaces.hasMoreElements()) {
					NetworkInterface n = interfaces.nextElement();
					String name = n.getName();
					if (!n.isLoopback() && name != null) {
						Network currentNetwork = getNetwork(name);
						currentNetwork.setPresent(true);
						if (n.isUp()) {
							InetAddress connectedAddress = checkConnected(n);
							if (connectedAddress != null) {
								NetworkCapabilities capabilities = getCapabilities(connectedAddress);
								if (capabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)) {
									if (internetNetwork != activeNetwork) {
										internetNetwork = currentNetwork;
										break;
									}
								} else if (availableNetwork != activeNetwork || availableNetwork == null) {
									availableNetwork = currentNetwork;
								}
							} else if (notAvailableNetwork != activeNetwork || notAvailableNetwork == null) {
								notAvailableNetwork = currentNetwork;
							}
						} else if (downNetwork != activeNetwork) {
							downNetwork = currentNetwork;
						}
					}
				}
			}

			if (internetNetwork == null) {
				for (Iterator<Entry<String, Network>> iterator = PollerConnectivityManager.this.networks.entrySet()
						.iterator(); iterator.hasNext();) {
					Entry<String, Network> entry = iterator.next();
					if (entry.getKey() != EMPTY_NETWORK && !entry.getValue().isPresent()) {
						iterator.remove();
					}
				}
			}

			updateNetworkState(internetNetwork, availableNetwork, notAvailableNetwork, downNetwork, activeNetwork);
		}

		private void updateNetworkState(@Nullable Network internetNetwork, @Nullable Network availableNetwork,
				@Nullable Network notAvailableNetwork, @Nullable Network downNetwork, Network activeNetwork) {
			updatePollingRate(availableNetwork != null, internetNetwork != null);
			// Set the best available network as active, if the current active
			// is the same, it will be filtered
			// out during the set.
			if (internetNetwork != null) {
				notify(internetNetwork, true, INTERNET_CONNECTED, activeNetwork);
			} else if (availableNetwork != null) {
				notify(availableNetwork, true, INTERNET_UNCONNECTED, activeNetwork);
			} else if (notAvailableNetwork != null) {
				notify(notAvailableNetwork, false, INTERNET_UNCONNECTED, activeNetwork);
			} else if (downNetwork != null) {
				notify(downNetwork, false, INTERNET_UNCONNECTED, activeNetwork);
			} else {
				if (activeNetwork != PollerConnectivityManager.this.networks.get(EMPTY_NETWORK)) {
					Network network = PollerConnectivityManager.this.networks.get(EMPTY_NETWORK);
					assert network != null; // since we added the empty network
											// in the constructor
					notify(network, false, INTERNET_UNCONNECTED, activeNetwork);
				}
			}
		}

		private void notify(Network newNetwork, boolean available, NetworkCapabilities capabilities,
				Network previousNetwork) {
			boolean willAvailableChanged = newNetwork.willChanged(available);
			boolean willCapabilitiesChanged = newNetwork.willChanged(capabilities);
			newNetwork.setAvailable(available);
			newNetwork.setCapabilities(capabilities);
			PollerConnectivityManager.this.activeNetwork = newNetwork;
			setActiveNetwork(newNetwork);
			if (newNetwork != previousNetwork) {
				// Notify previous owner.
				if (previousNetwork.getCapabilities() == INTERNET_CONNECTED) {
					previousNetwork.setCapabilities(INTERNET_UNCONNECTED);
					notifyNetworkCallbacks(previousNetwork, INTERNET_UNCONNECTED);
				}

				if (previousNetwork.isAvailable()) {
					previousNetwork.setAvailable(false);
					notifyNetworkCallbacks(previousNetwork, false);
				}
			}
			if (available) {
				if (willAvailableChanged) {
					notifyNetworkCallbacks(newNetwork, available);
				}
				if (willCapabilitiesChanged) {
					notifyNetworkCallbacks(newNetwork, capabilities);
				}
			} else if (newNetwork == previousNetwork) {
				if (willCapabilitiesChanged) {
					notifyNetworkCallbacks(newNetwork, capabilities);
				}
				if (willAvailableChanged) {
					notifyNetworkCallbacks(newNetwork, available);
				}
			}
		}

		private void updatePollingRate(boolean avaialble, boolean internet) {
			if (internet) {
				if (PollerConnectivityManager.this.currentPeriod != PollerConnectivityManager.this.periodWhenInternet) {
					startPolling(PollerConnectivityManager.this.periodWhenInternet,
							PollerConnectivityManager.this.periodWhenInternet);
				}
			} else if (avaialble
					&& PollerConnectivityManager.this.currentPeriod != PollerConnectivityManager.this.periodWhenAvailable) {
				startPolling(PollerConnectivityManager.this.periodWhenAvailable,
						PollerConnectivityManager.this.periodWhenAvailable);
			} else if (!avaialble
					&& PollerConnectivityManager.this.currentPeriod != PollerConnectivityManager.this.periodWhenLost) {
				startPolling(PollerConnectivityManager.this.periodWhenLost,
						PollerConnectivityManager.this.periodWhenLost);
			}
		}

		private Network getNetwork(String networkInterface) {
			Network network = PollerConnectivityManager.this.networks.get(networkInterface);
			if (network == null) {
				network = new Network(networkInterface);
				PollerConnectivityManager.this.networks.put(networkInterface, network);
			}
			return network;
		}

		private @Nullable InetAddress checkConnected(@Nullable NetworkInterface n) {
			InetAddress connectedAddress = null;
			if (n != null) {
				Enumeration<InetAddress> ee = n.getInetAddresses();
				while (ee.hasMoreElements()) {
					InetAddress inetAddress = ee.nextElement();
					if (NetUtil.isValidInetAdress(inetAddress)) {
						connectedAddress = inetAddress;
						break;
					}
				}
			}
			return connectedAddress;
		}
	}

	/**
	 * Gets the capabilities of a network.
	 *
	 * @param connectedAddress
	 *            the address to get the capabilities from, cannot be <code>null</code>.
	 * @return the capabilities of the address.
	 */
	protected NetworkCapabilities getCapabilities(InetAddress connectedAddress) {
		return INTERNET_UNCONNECTED;
	}

	/**
	 * Isolate for testing.
	 *
	 * @return all the interface available.
	 * @throws SocketException
	 *             if an I/O exception occurs.
	 */
	protected @Nullable Enumeration<NetworkInterface> getNetworkInterfaces() throws SocketException {
		return NetworkInterface.getNetworkInterfaces();
	}

	/**
	 * Isolate for testing.
	 *
	 * @param name
	 *            the name of the interface, cannot be <code>null</code>.
	 * @return the interface with a name.
	 * @throws SocketException
	 *             if an I/O exception occurs.
	 */
	protected @Nullable NetworkInterface getNetworkInterface(String name) throws SocketException {
		return NetworkInterface.getByName(name);
	}

}
